from nexilum.connect_to import connect_to, auth, login, logout
from nexilum.nexilum import Nexilum
from exception_nexilum.nexilum_error import Nexilum_error

__version__ = '0.0.5'
__all__ = ['Nexilum', 'connect_to', 'Nexilum_error', 'auth', 'login', 'logout']
